@@ -1,6 +1,6 @@
 ###############################################################################
 #
-# This file copyright (c) 2001-2011 Randy J. Ray, all rights reserved
+# This file copyright (c) 2001-2014 Randy J. Ray, all rights reserved
 #
 # Copying and distribution are permitted under the terms of the Artistic
 # License 2.0 (http://www.opensource.org/licenses/artistic-license-2.0.php) or
@@ -47,14 +47,15 @@
 #                   timeout
 #                   server_fault
 #
-#   Libraries:      AutoLoader
-#                   HTTP::Daemon (conditionally)
+#   Libraries:      HTTP::Daemon (conditionally)
 #                   HTTP::Response
 #                   HTTP::Status
 #                   URI
+#                   Scalar::Util
 #                   RPC::XML
 #                   RPC::XML::ParserFactory
 #                   RPC::XML::Procedure
+#                   Compress::Raw::Zlib is used if available
 #
 #   Global Consts:  $VERSION
 #                   $INSTALL_DIR
@@ -67,17 +68,19 @@ package RPC::XML::Server;
 use 5.008008;
 use strict;
 use warnings;
-use vars qw($VERSION $INSTALL_DIR %FAULT_TABLE  @XPL_PATH
+use vars qw($VERSION $INSTALL_DIR %FAULT_TABLE  @XPL_PATH %CLASS_MAP
             $IO_SOCKET_SSL_HACK_NEEDED $COMPRESSION_AVAILABLE);
 
 use Carp qw(carp croak);
 use File::Spec;
+use File::Temp;
 use IO::Handle;
+use Module::Load;
+use Scalar::Util 'blessed';
 
 use HTTP::Status;
 use HTTP::Response;
 use URI;
-use Scalar::Util 'blessed';
 
 use RPC::XML;
 use RPC::XML::ParserFactory;
@@ -96,10 +99,8 @@ BEGIN
     $IO_SOCKET_SSL_HACK_NEEDED = 1;
 
     # Check for compression support
-    if (! eval { require Compress::Zlib; $COMPRESSION_AVAILABLE = 'deflate'; })
-    {
-        $COMPRESSION_AVAILABLE = q{};
-    }
+    $COMPRESSION_AVAILABLE =
+        (eval { load Compress::Zlib; 1; }) ? 'deflate' : q{};
 
     # Set up the initial table of fault-types and their codes/messages
     %FAULT_TABLE = (
@@ -108,9 +109,16 @@ BEGIN
         badsignature => [ 201 => 'Method signature error: %s' ],
         execerror    => [ 300 => 'Code execution error: %s' ],
     );
+
+    # This is used by add_method to map "types" to instantiation classes
+    %CLASS_MAP = (
+        method    => 'RPC::XML::Method',
+        procedure => 'RPC::XML::Procedure',
+        function  => 'RPC::XML::Function',
+    );
 }
 
-$VERSION = '1.67';
+$VERSION = '1.73';
 $VERSION = eval $VERSION; ## no critic (ProhibitStringyEval)
 
 ###############################################################################
@@ -138,7 +146,12 @@ sub new ## no critic (ProhibitExcessComplexity)
         $srv_name
     );
 
-    $class = ref($class) || $class;
+    # Don't accept a blessed value for $class
+    if (ref $class)
+    {
+        return __PACKAGE__ . '::new: Must be called as a static method';
+    }
+
     $self = bless {}, $class;
 
     $srv_version = delete $args{server_version} || $self->version;
@@ -158,7 +171,7 @@ sub new ## no critic (ProhibitExcessComplexity)
         $port  = delete $args{port}  || q{};
         $queue = delete $args{queue} || 5;
         $http  = HTTP::Daemon->new(
-            Reuse => 1,
+            ReuseAddr => 1,
             ($host  ? (LocalHost => $host)  : ()),
             ($port  ? (LocalPort => $port)  : ()),
             ($queue ? (Listen    => $queue) : ())
@@ -204,6 +217,9 @@ sub new ## no critic (ProhibitExcessComplexity)
         $args{parser} ? @{delete $args{parser}} : ()
     );
 
+    # Add the basic paths (content of @XPL_PATH) to our local XPL path
+    push @{$self->{__xpl_path}}, @XPL_PATH;
+
     # Set up the default methods unless requested not to
     if (! delete $args{no_default})
     {
@@ -392,6 +408,8 @@ sub xpl_path
 #                   $self     in      ref       Object to add to
 #                   $meth     in      scalar    Hash ref of data or file name
 #
+#   Globals:        %CLASS_MAP
+#
 #   Returns:        Success:    $self
 #                   Failure:    error string
 #
@@ -416,11 +434,22 @@ sub add_method
     }
     elsif (ref $meth eq 'HASH')
     {
+        # Make a copy of the contents of $meth, so we don't make permanent
+        # changes:
+        my %meth_copy = map { $_ => $meth->{$_} } (keys %{$meth});
+
         # If the type of this method is not set, default to "method". The
         # add_procedure and add_function calls should set this as needed.
-        $meth->{type} ||= 'method';
-        my $class = 'RPC::XML::' . ucfirst $meth->{type};
-        $meth = $class->new($meth);
+        my $type = delete $meth_copy{type} || 'method';
+
+        if (! (my $class = $CLASS_MAP{lc $type}))
+        {
+            return "$me: Unknown type: $type";
+        }
+        else
+        {
+            $meth = $class->new(\%meth_copy);
+        }
     }
     elsif (! (blessed $meth and $meth->isa('RPC::XML::Procedure')))
     {
@@ -463,7 +492,7 @@ sub add_procedure
 
 ###############################################################################
 #
-#   Sub Name:       add_procedure
+#   Sub Name:       add_function
 #
 #   Description:    This filters through to add_method, but if the passed-in
 #                   value is a hash reference forces the "type" to be
@@ -512,10 +541,10 @@ sub method_from_file
     if (! File::Spec->file_name_is_absolute($file))
     {
         my $path;
-        for my $dir (@{$self->xpl_path}, @XPL_PATH)
+        for my $dir (@{$self->xpl_path})
         {
             $path = File::Spec->catfile($dir, $file);
-            if (-e $path)
+            if (-f $path)
             {
                 $file = File::Spec->canonpath($path);
                 last;
@@ -616,7 +645,8 @@ sub server_loop ## no critic (RequireArgUnpacking,ProhibitExcessComplexity)
 
     if ($self->{__daemon})
     {
-        my ($conn, $req, $resp, $reqxml, $respxml, $exit_now, $timeout);
+        my ($conn, $req, $resp, $reqxml, $respxml, $exit_now, $timeout,
+            $eval_return);
 
         my %args = @_;
 
@@ -652,7 +682,17 @@ sub server_loop ## no critic (RequireArgUnpacking,ProhibitExcessComplexity)
             }
             $conn->timeout($self->timeout);
             $self->process_request($conn);
-            $conn->close;
+
+            $eval_return = eval {
+                local $SIG{PIPE} = sub { die "server_loop: Caught SIGPIPE\n"; };
+                $conn->close;
+                1;
+            };
+            if ((! $eval_return) && $@)
+            {
+                warn "Cannot close connection: $@\n";
+            }
+
             undef $conn;    # Free up any lingering resources
         }
 
@@ -787,9 +827,14 @@ sub process_request ## no critic (ProhibitExcessComplexity)
     );
 
     my $me = ref($self) . '::process_request';
-    if (! ($conn && ref $conn))
+    if (! $conn)
     {
+        # Maintain compatibility with Net::Server 0.99, which does not pass
+        # the connection object at all:
         $conn = $self->{server}->{client};
+    }
+    if (ref($conn) =~ /^Net::Server::Proto/)
+    {
         bless $conn, 'HTTP::Daemon::ClientConn';
         ${*{$conn}}{'httpd_daemon'} = $self;
 
@@ -829,6 +874,7 @@ sub process_request ## no critic (ProhibitExcessComplexity)
             # Get a XML::Parser::ExpatNB object
             $parser = $self->parser->parse();
 
+            $do_compress = 0; # in case it was set for a previous response
             if (($req->content_encoding || q{}) =~ $self->compress_re)
             {
                 if (! $self->compress)
@@ -1069,7 +1115,7 @@ sub process_request ## no critic (ProhibitExcessComplexity)
             else
             {
                 # Treat the content strictly in-memory
-                utf8::downgrade($buf = $respxml->as_string);
+                utf8::encode($buf = $respxml->as_string);
                 if ($do_compress)
                 {
                     $buf = Compress::Zlib::compress($buf);
@@ -2290,7 +2336,7 @@ XPL specifies a namespace, see L<RPC::XML::Procedure|RPC::XML::Procedure>). If
 the code element of a procedure/method is passed in as a direct code reference
 by one of the other syntaxes allowed by the constructor, the package may well
 be different. Thus, routines should strive to be as localized as possible,
-independant of specific namespaces. If a group of routines are expected to work
+independent of specific namespaces. If a group of routines are expected to work
 in close concert, each should explicitly set the namespace with a C<package>
 declaration as the first statement within the routines themselves.
 
